//pseudo sticky class
//creates a basic sticky note (resizable area with a textField in it)
//name must be equal to the variable name!
function Sticky(name)
{
    //a few constants
    const WIDTH =  375;     //default width
    const HEIGHT = 375;     //default height
    
    const BG_PATH = "Resources/Images/peach.png";   //path to bg 1x1 pixel bg color
    const RA_PATH = "Resources/Images/resize.png";  //path to resize anchor image
    const CB_UP_PATH = "Resources/Images/close-up.png";   //path to close button image
    const CB_OVER_PATH = "Resources/Images/close-over.png";   //path to close button image
    const MB_UP_PATH = "Resources/Images/minimize-up.png"; //minimize button up
    const MB_OVER_PATH = "Resources/Images/minimize-over.png"; //minimize button up
    
    const TB_PATH           = "Resources/Images/topbar.png";   //path to title bar bg color
    //const TB_BORDER_PATH    = "Resources/Images/black.png";          //path to title bar border color
    const TB_VOFFSET        = 19;                                   //height of the titlebar
    const TB_FONT_FAM       = "Gill Sans";                          //font of the title bar (when collapsed)
    const TB_FONT_SIZE      = 10;                                    //font size of title bar (when collapsed)
    
    const TA_MARGIN     = 10;           //margin for text area (left and right margins)
    const TA_FONT_FAM   = "Gill Sans";  //text area font
    const TA_FONT_SIZE  = 12;           //text area size
    
    const MIN_WIDTH     = 150 + TA_MARGIN;               //minimum resize width
    const MIN_HEIGHT    = 150 + TB_VOFFSET + TA_MARGIN;  //minimum resize height

    //give is a name
    this.myName = name;
    
    //create our window
    this.myWin = new Window(this.myName + "_win");
    this.myWin.width = WIDTH;
    this.myWin.height = this.expandedHeight = HEIGHT;
    this.myWin.visible = true;
    this.myWin.shadow = true;
    this.window = this.myWin;

    //create our background
    this.myBG = new Image(this.myWin);
    this.myBG.hOffset = this.myBG.vOffset = 0;
    this.myBG.src = BG_PATH;
    this.myBG.zOrder = 0;
    
    //create our title bar & title bar border (height remains constant)
    this.myTB = new Image(this.myWin);
    this.myTB.src = TB_PATH;
    this.myTB.hOffset = this.myTB.vOffset = 0;
    this.myTB.height = TB_VOFFSET;
    this.myTB.zOrder = 4;
    this.myTB.onMultiClick = this.myName + ".titleBarMultiClick()";
    
    this.collapsed = false;
    
    //create our minimize button
    this.myMB = new Image(this.myWin);
    this.myMB.src = MB_UP_PATH;
    this.myMB.vOffset = 0;
    this.myMB.tracking = "rectangle";
    this.myMB.onMouseUp = this.myName + ".titleBarMultiClick()";
    this.myMB.onMouseEnter = this.myName + ".onMBMouseEnter()";
    this.myMB.onMouseExit = this.myName + ".onMBMouseExit()";
    this.myMB.zOrder = 10;
    
    //when mouse enters the minimize button
    this.onMBMouseEnter = function()
    {
    	this.myMB.src = MB_OVER_PATH;
    }
    
    //when mouse exits the minimize button
    this.onMBMouseExit = function()
    {
    	this.myMB.src = MB_UP_PATH;
    }
    
    //create our close button
    this.myCB = new Image(this.myWin);
    this.myCB.src = CB_UP_PATH;
    this.myCB.hOffset = this.myCB.vOffset = 0;
    this.myCB.tracking = "rectangle";
    this.myCB.zOrder = 6;
    this.myCB.onMouseUp = this.myName + ".close()";
    this.myCB.onMouseEnter = this.myName + ".onCBMouseEnter()";
    this.myCB.onMouseExit = this.myName + ".onCBMouseExit()";
    
    //when mouse enters the close button
    this.onCBMouseEnter = function()
    {
    	this.myCB.src = CB_OVER_PATH;
    }
    
    //when mouse exits the close button
    this.onCBMouseExit = function()
    {
    	this.myCB.src = CB_UP_PATH;
    }
    
    //when close button is released
    this.close = function()
    {
        var response = alert("Are you sure you want to delete this sticky? It cannot be recovered once deleted.", "Delete", "Cancel");
						   
        if (response == 2)
            return false;
		
		this.myWin.visible = false;
		delete this.myWin;
		
		//remove our data
		var myPath = system.widgetDataFolder + "/" + this.myName + "/"; //path to folder that holds our data
		runCommand("rm -r '" + myPath + "'" );
		
		return true;
    }
    
    //when we double click the title bar we need a collapse/expand
    this.titleBarMultiClick = function()
    {
        if( this.collapsed )
            this.expand();
        else
            this.collapse();
        
        updateNow();
    }
    
    //collapse (minimize) our sticky
    this.collapse = function()
    {
        this.collapsed = true;
        
        //hide everything but the title bar
        this.expandedHeight = this.myWin.height; //so we can expand later
        this.myWin.height = TB_VOFFSET;
        
        //add text to title bar
        this.myTB_TA = new Text(this.myWin);
        this.myTB_TA.height = TB_VOFFSET;
        this.myTB_TA.hOffset = 65;
        this.myTB_TA.vOffset = TB_FONT_SIZE;
        this.myTB_TA.width = this.myWin.width - this.myTB_TA.hOffset - 20;
        this.myTB_TA.font = TB_FONT_FAM;
        this.myTB_TA.size = TB_FONT_SIZE;
        this.myTB_TA.data = this.myTA.data;
    }
    
    //expand our sticky
    this.expand = function()
    {
        this.collapsed = false;
        
        //show stuff again
        this.myWin.height = this.expandedHeight;
        
        //remove text in title bar
        delete this.myTB_TA;
    }
    
    //create our text area
    this.myTA = new TextArea(this.myWin);
    this.myTA.vOffset = 65;
    this.myTA.scrollbar = false;
    this.myTA.font = TA_FONT_FAM;
    this.myTA.size = TA_FONT_SIZE;
    this.myTA.zOrder = 3;
    
    //create our timer (used for resizing)
    this.resizeTimer = new Timer();
    this.resizeTimer.ticking = false;
    this.resizeTimer.interval = .05;
    this.resizeTimer.onTimerFired = this.myName + ".resizeTimerFired();";
    
    //when this is firing it means we are resizing
    this.resizeTimerFired = function()
    {
        //define mousePos in here because of some quirky mouse event stuff...
        if( this.mousePos == null )
        {
            this.mousePos = {x:system.event.screenX, y:system.event.screenY};
            return;
        }
        
        //things are backwards in mouse event land...
        var deltaX = -(this.mousePos.x - system.event.screenX);
        var deltaY = -(this.mousePos.y - system.event.screenY);
        
        if( deltaX == 0 && deltaY == 0) //no changes
            return;
        
        //resize stuff
        var nextWidth = this.myWin.width + deltaX;
        var nextHeight = this.myWin.height + deltaY;
        
        //don't let it go smaller than the minimum
        if(nextWidth < MIN_WIDTH || nextHeight < MIN_HEIGHT)
            return;
        
        this.myWin.width = nextWidth;
        this.myWin.height = this.expandedHeight = nextHeight;
        this.draw();
        
        this.mousePos = {x:system.event.screenX, y:system.event.screenY};
    }
    
    //create our resize anchor
    this.myRA = new Image(this.myWin);
    this.myRA.src = RA_PATH;
    this.myRA.tracking = "rectangle";
    
    //resize event handlers
    this.myRA.onMouseDown = this.myName + ".resizeMouseDown();";
    this.myRA.onMouseUp = this.myName + ".resizeMouseUp();";
    
    this.resizeMouseDown = function()
    {
        this.resizeTimer.ticking = true;    //start resizing
    }
    
    this.resizeMouseUp = function()
    {
        this.mousePos = null;
        this.resizeTimer.ticking = false;    //stop resizing
    }
    
    //preferences functions
    this.loadPrefs = function()
    {
        var myPath = system.widgetDataFolder + "/" + this.myName + "/"; //path to folder that holds our data
        
        //see if this is a first time load... if it is then create all required files
        var firstLoad = !filesystem.itemExists( myPath );
        
        if( firstLoad )
        {
            //default values are already set in constructor... just create our folder & initial files
            runCommand("mkdir '" + myPath + "'" );    //create our folder
            filesystem.writeFile(myPath + "text.txt", "");
            filesystem.writeFile(myPath + "position.txt", this.myWin.width + "\n" + this.expandedHeight + "\n" +  this.myWin.hOffset + "\n" + this.myWin.vOffset);
            return;
        }
        
        //load our stuffs
        var positions_arr = filesystem.readFile( myPath + "position.txt", true );
        this.myWin.width = positions_arr[0];
        this.myWin.height = this.expandedHeight = positions_arr[1];
        this.myWin.hOffset = positions_arr[2];
        this.myWin.vOffset = positions_arr[3];
        this.collapsed = positions_arr[4] == "true";
        
        this.myTA.data = filesystem.readFile( myPath + "text.txt" );
    }
    
    this.loadPrefs();
    
    this.savePrefs = function()
    {
        var myPath = system.widgetDataFolder + "/" + this.myName + "/"; //path to folder that holds our data
        filesystem.writeFile(myPath + "text.txt", this.myTA.data);
        filesystem.writeFile(myPath + "position.txt", this.myWin.width + "\n" + this.expandedHeight + "\n" +  this.myWin.hOffset + "\n" + this.myWin.vOffset + "\n" + this.collapsed);
    }
    
    //draws and sizes our objects
    this.draw = function()
    {
        //draw our background
        this.myBG.width = this.myWin.width;
        this.myBG.height = this.myWin.height;
        
        //draw our textarea
        this.myTA.hOffset = TA_MARGIN;
        this.myTA.width = this.myWin.width - (2 * TA_MARGIN);
        this.myTA.height = this.myWin.height - this.myTA.vOffset - 50;
        
        //draw our resize anchor
        this.myRA.vOffset = this.myWin.height - this.myRA.height;
        this.myRA.hOffset = this.myWin.width - this.myRA.width;
        
        //draw our title bar
        this.myTB.width = this.myWin.width;
        
        //draw minimize button
        this.myMB.hOffset = this.myWin.width - this.myMB.width;
    }
    
    //initial draw
    this.draw();
}